# Converts output from the dot factory to binary format
# Starts at 33 - space is not included so is special
# Format
# <index start 2 bytes><body start 2 bytes>
# index: <width 2bytes><height in bytes 2 bytes><offset 2 bytes>......
# body: <character information stored as bytes>
# with char from ! to ~ index offset starts at 0 and body starts at 94 * 3 * 2 = 564
#
# Limitations
# 1) only works with correct settings for sdd1306 see OutputConfigs.xml
# 2) dotfactory version 0.1.4
# 3) Must have full charater set from '!' to '~'

import sys
import os.path
PREAMBLESIZE = 4 # in bytes

def ConvertFont(infile,outfile):
    of = open(outfile, 'wb')
    lines = tuple(open(infile, 'r'))  # get the lot
    # positions
    index=0
    body = 0
    # this is the preamble
    of.write((PREAMBLESIZE).to_bytes(2,'big')) # index start
    of.write((564+PREAMBLESIZE).to_bytes(2,'big')) # body start
    
    for i, line in enumerate(lines):
        if 'Character bitmaps' in line:
            body = i + 4
        if 'FONT_CHAR_INFO' in line:
            index = i + 2
    # index
    for line in lines[index:]:
        first = line.find('{')
        if first >= 0:
            first +=1
            second = line.find('}')
            if second >= 0:
                cell = line[first:second].split(',')
#                print(cell)
                b1 = int(cell[0]).to_bytes(2,'big')
                b2 = int(cell[1]).to_bytes(2,'big')
                b3 = int(cell[2]).to_bytes(2,'big')
                of.write(b1)    
                of.write(b2)    
                of.write(b3)    
    # body
#    count = 0
    for line in lines[body:index]:
#        print(line)
        first = line.find('0x')
        if first >= 0:
            first -= 1
            second = line.find(', //')
            if second >= 0:
                cell = line[first:second].strip().split(',')
#                print(cell)                
                for b in cell:
                    of.write(int(b,16).to_bytes(1,'big'))                
#        count +=1
#        if count > 338: break           
#   print(body,lines[index])    
    of.close()

def ConvertBitmap(infile,outfile):
    print('converting bitmap as c file from {} to {}'.format(infile,outfile))
    of = open(outfile, 'wb')
    lines = tuple(open(infile, 'r'))  # get the lot

    # extract width and height from end of file
    first = False
    for i, line in enumerate(lines):
        if '0x' in line: # first one
            if not first:
                body = i
                first = True
        if '}' in line:
            endofbody = i - 1         
        if 'WidthPixels' in line:
            ipart = line.split('=')[1].strip()
            width = int(ipart[:-1]) # drop semicolon
        if 'HeightPages' in line:
            ipart = line.split('=')[1].strip()
            height = int(ipart[:-1]) # drop semicolon

    # this is the preamble
    of.write((width).to_bytes(2,'big')) # index start
    of.write((height).to_bytes(2,'big')) # body start
    
    # main body
#    print(body,endofbody)
#    print(lines[endofbody])
    for line in lines[body:endofbody]:
        nbytes = line.strip().split(',')
#        print(len(nbytes),nbytes)
        if len(nbytes) > 5:
            for b in nbytes:
                if len(b) > 3:
                    of.write(int(b,16).to_bytes(1,'big'))    
    
    of.close()

def help():
    ht='''
This will convert a .c file generated by Dot Factory version 0.1.4
to a binary font file suitable for use with ssd1036_i2c.py. Use save_as
in Dot Factory to generate a .c and .h file, the .h file is not needed.
Also in the settings rotate by 90 degrees and in 'Descriptors' Char height
should be set to bytes.
Arial 10 output will look like this, on the diplay this is 2 bytes high
by 7 pixels wide
	// @280 'A' (7 pixels wide)
	0x18, 0x00, //    ##        
	0x07, 0x00, //      ###     
	0x02, 0xE0, //       # ###  
	0x02, 0x10, //       #    # 
	0x02, 0xE0, //       # ###  
	0x07, 0x00, //      ###     
	0x18, 0x00, //    ## 
Supply the input file first (normally .c) followed by the output file (normally .fon)
Bitmap Example python3 convert.py -b 127x63.c 127.63.bin
Ref:
http://www.byvac.com
http://www.eran.io/the-dot-factory-an-lcd-font-and-image-generator/
    
    '''
    print(ht)
if __name__ == '__main__':
    if len(sys.argv) == 3:
        if os.path.isfile(sys.argv[1]):
            ConvertFont(sys.argv[1],sys.argv[2])
        else:        
            print('{} does not exist'.format(sys.argv[1]))
            help() 
    elif len(sys.argv) == 4:
        if sys.argv[1] == '-b':
            if os.path.isfile(sys.argv[2]):
                ConvertBitmap(sys.argv[2],sys.argv[3])
            else:        
                print('{} does not exist'.format(sys.argv[2]))
                help()
    else:
        help()         
                            

